#ifndef __EXT2Dir_h__
#define __EXT2Dir_h__

#include "fs/FSResult.H"
#include "fs/FSDirImpl.H"
#include "fs/FSObj.H"
#include "fs/ext2/EXT2File.H"

#include <iostream>

/**
 * This class implements the interface of FSDIrImpl. It implements read and 
 * write acces to a directory. Operation on references are forwarded to EXT2Obj.
 */
class EXT2Dir : public FSDirImpl {
  /** pointer to the interface of the filesystem */
  EXT2FS *fs;
public:
  /**
   * Default constructor of the EXT2Dir class.
   *
   * @param filesystem pointer to the filesystem
   * @param inodenr number of the starting inode
   */
  EXT2Dir(EXT2FS *filesystem, unsigned long inodenr);
  ~EXT2Dir() {}

  /**
   * This method identifies this object as a directory
   *
   * @return Returns the type of the object identifying it as a directory.
   */
  FSObj::Type type() { return FSObj::Directory; }

  /**
   * Creates a new file by instanziating an new EXT2File object.
   *  
   * @param name name of the new file
   * @param file pointer to the instanziated file object
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult create(const char* name, FSFileImpl *&file);      

  /**
   * Looks for the entry with the given name and checks its type. If the entry
   * is a directory an instanze of this directory is returned.
   * 
   * @param name name of the directory
   * @param dir instanze of the directory
   * @return Returns the error status of the filesystem as an FSResult instanze.
   *    <ul>
   *      <li>FSResult::NoError default if there has not been an error</li>
   *      <li>FSResult::NoDirectory if found entry is no directory</li>
   *      <li>FSResult::FileNotFound if an according entry could not be found</li>
   *      <li>FSResult::UninitializedObj if no memory to instanziate the object was available</li>
   *      <li>FSResult::FsSpecificError otherwise</li>
   *    </ul>
   */
  FSResult get_entry(const char* name, FSDirImpl *&dir);

  /**
   * Looks for the entry with the given name and checks its type. If the entry
   * is a file an instanze of this file is returned.
   * 
   * @param name name of the file
   * @param file instanze of the file
   * @return Returns the error status of the filesystem as an FSResult instanze.
   *    <ul>
   *      <li>FSResult::NoError default if there has not been an error</li>
   *      <li>FSResult::FileNotFound if an according entry could not be found</li>
   *      <li>FSResult::UninitializedObj if no memory to instanziate the object was available</li>
   *      <li>FSResult::FsSpecificError otherwise</li>
   *    </ul>
   */
  FSResult get_entry(const char* name, FSFileImpl *&file);

  /**
   * Creates a new subdirectory by instanziating an new EXT2Dir object.
   *  
   * @param name name of the new directory
   * @param dir pointer to the instanziated dir object
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult mkdir(const char* name, FSDirImpl *&dir);

  /**
   * Moves an entry from the directory to an other directory.
   *
   * @param name name of the file/dir to be moved
   * @param location interface of the destination directory
   * @param new_name new name of the entry
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult move(const char *name, FSDirImpl *location, const char* new_name = 0);

  /**
   * Creates a new hardlink for a given source.
   * 
   * @param name source of the link (file/directory)
   * @param destination interface of the destination directory
   * @param new_name name of the link
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult link(const char *name, FSDirImpl *destination, const char* new_name = 0); 

  /**
   * Reads the next entry from the directory.
   *
   * @param buffer buffer to read entry to
   * @param buffer_size size of the given buffer
   * @param type type (file/dir) of the entry (will be modified by the operation)
   * @return Returns the error status of the filesystem as an FSResult instanze.
   *    <ul>
   *      <li>FSResult::NoError default if there has not been an error</li>
   *      <li>FSResult::BufferTooSmall if given buffer is to small</li>
   *      <li>FSResult::EndOfFile if no entries are left</li>
   *      <li>FSResult::FsSpecificError otherwise</li>
   *    </ul>
   */
  FSResult read_entry(char* buffer, int &buffer_size, FSObj::Type &type);

  /**
   * Removes an entry from the directory by releasing the allocated structures.
   *
   * @param name name of the file/dir to be removed
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult remove(const char *name);

  /**
   * @return Returns the error status of the filesystem as an FSResult instanze 
   *         (default is FSResult::NoError).
   */
  FSResult reset();

  FSResult attribute_count(const char* name, int &count);
  FSResult attribute_name(const char* name, unsigned int n, const char *&attr_name);
  FSResult attribute_description(const char *name, const unsigned int n, const char *&description);

  /**
   * Reads an attribute from a given entry (file, dir, etc.).
   *
   * @param name name of the file/dir
   * @param attr_name name of the attribute
   * @param val buffer to be read to
   * @param size size of the given buffer
   * @return Returns the error status of the filesystem as an FSResult instanze.
   *    <ul>
   *      <li>FSResult::NoError default if there has not been an error</li>
   *      <li>FSResult::FileNotFound if name could not be found</li>
   *      <li>FSResult::BufferTooSmall if given buffer is to small</li>
   *      <li>FSResult::AttribUnknown if name of attribute is unknown</li>
   *      <li>FSResult::FsSpecificError otherwise</li>
   *    </ul>
   */
  FSResult get_attribute(const char *name, const char* attr_name, char *val, int& size) ;

  /**
   * @return Returns the error status of the filesystem as an FSResult instanze.
   *    <ul>
   *      <li>FSResult::NoError default if there has not been an error</li>
   *      <li>FSResult::FileNotFound if name could not be found</li>
   *      <li>FSResult::AttribUnknown if name of attribute is unknown</li>
   *      <li>FSResult::Readonly if filesystem is readonly</li>
   *      <li>FSResult::InvalidValue if the new value of the attribute is out of scope</li>
   *      <li>FSResult::NotChangeable if changing the attribute is not allowed</li>
   *      <li>FSResult::FsSpecificError otherwise</li>
   *    </ul>
   */
  FSResult set_attribute(const char *name, const char* attr_name, const char *val) ;
};

#endif /* __EXT2Dir_h__ */
