#ifndef __EXT2FS_h__
#define __EXT2FS_h__

#include "fs/FS.H"
#include "fs/FSResult.H"
#include "fs/device/block/BlockDevice.H"

class EXT2Dir;

/**
 * Class EXT2FS implements the filesystem interface for the ext2-filesystem.
 * EXT2FS uses itable to manage acces to the filesystem
 * get_itable wird von EXT2Dir und EXT2File verwendet, um Zugriff auf
 * die Manager (InodeTable, Blockgrp, Superblock und BlockDev) zu
 * bekommen.
 */
class EXT2FS : public FS 
{
private:
  friend class EXT2Dir;
  friend class EXT2File;

  /** number of last occured error */
  int	errno;
  /** device containing the data of the filesystme */
  BlockDevice* device;
  /** pointer to the root-directory of the filesystem */
  EXT2Dir* root_dir;  
public:
  /**
   * Default constructor checks if the given BlockDevice is proper and creates
   * an inodetable as well as a pointer to the root-directory(inode number 2).
   * Sets the error number of the filesystem accordingly:
   * <ul>
   *   <li>EXT2E_FS_NULL_DEVICE if no BlockDevice is given</li>
   *   <li>EXT2E_FS_INIT_FAILED if there is no memory for the inodetable</li>
   *   <li>EXT2E_FS_ROOT_FAILED if there is no memory for the root-directory</li>
   * </ul>
   *
   * @param device Blockdevice that contains the image of the filesystem
   */
  EXT2FS(BlockDevice *device);

  /**
   * Destructor of the class deletes the instanze of the root-directory and the
   * inodetable.
   */
  ~EXT2FS();

  /**
   * Checks if there have been any errors during an operation (initialisation).
   *
   * @return Returns the error status of the filesystem as an FSResult instanze.
   *    <ul>
   *      <li>FSResult::NoError default if there has not been an error</li>
   *      <li>FSResult::UninitializedObj if no BlockDevice was given or there is
   *          not enough memory to creat the inodetable or the root-directory
   *      </li>
   *      <li>FSResult::InvalidFS if given filesystem is not an ext2</li>
   *      <li>FSResult::FsSpecificError if BlockDevice could not be initialised 
   *          or read, etc.
   *      </li>
   *    </ul>
   */
  FSResult check();

  /**
   * Synchronises the inodetable by writing changes and chached data to the 
   * filesystem.
   *
   * @return Returns the error status of the filesystem as an FSResult instanze.
   *    <ul>
   *      <li>FSResult::NoError default if there has not been an error</li>
   *      <li>FSResult::FsSpecificError if there has been an error</li>
   *    </ul>
   */
  FSResult sync();

  /**
   * Gets the pointer to the root-directory.
   *
   * @return Returns a pointer to the FSDirImpl interface of the root-directory.
   */
  FSDirImpl* root();
  
  /**
   * Get the number of attributes for the filesystem.
   * 
   * @return Allways returns FSResult::NoError.
   */
  virtual FSResult attribute_count(int &count);

  /**
   * Get the name of an attribute.
   * 
   * @param n number of the attribute
   * @param name attribute name (will be modified by the operation)
   * @return Allways returns FSResult::NoError.
   */
  virtual FSResult attribute_name(unsigned int n, const char *&name);

  /**
   * Gets the description of an attribute.
   * 
   * @param n number of the attribute
   * @param name attribute description (will be modified by the operation)
   * @return Allways returns FSResult::NoError.
   */
  virtual FSResult attribute_description(unsigned int n, const char *&description);

  /**
   * Get the value of the specified attribute.
   *
   * @param name name of the attribute
   * @param val buffer that will store the value of the attribute
   * @param size size of the buffer (will be modified by the operation)
   * @return Returns the error status of the filesystem as an FSResult instanze.
   *    <ul>
   *      <li>FSResult::NoError default if there has not been an error</li>
   *      <li>FSResult::BufferTooSmall if size of the buffer is to small</li>
   *      <li>FSResult::AttribUnknown if name of the attribute is unknown</li>
   *    </ul>
   */
  virtual FSResult get_attribute(const char* name, char *val, int& size);

  /**
   * Sets the value of the specified attribute.
   * 
   * @param name name of the attribute
   * @param val new value of the attribute
   *
   * @return Returns the error status of the filesystem as an FSResult instanze.
   *    <ul>
   *      <li>FSResult::NoError default if there has not been an error</li>
   *      <li>FSResult::InvalidValue if the new value of the attribute is out of scope</li>
   *      <li>FSResult::AttribUnknown if name of the attribute is unknown</li>
   *    </ul>
   */
  virtual FSResult set_attribute(const char* name, const char *val);
};

#endif /* __EXT2FS_h__ */
