#include "fs/ext2/attributs.h"
#include "fs/FSResult.H"

/**
 * Converts a string of the format rwxrwxrwx+sticky+setgid+setuid 
 * into an int value.
 * 
 * @param val string that will be converted
 * @param perm int value created from string (will be modified by operation)
 * @return Returns the status of the operation as an FSResult 
 *         instanze (FSResult::NoError if no error occured or
 *         FSResult::InvalidValue if string is illegal)
 */
FSResult str2perm(const char *val, unsigned short &perm) 
{
  unsigned short h=0;
  if (strlen(val)!=9) return FSResult::InvalidValue;
  
  if (val[8]=='x') h+=01;
  else if (val[8]=='t') h+=01+01000;
  else if (val[8]=='T') h+=01000;
  else if (val[8]!='-') return FSResult::InvalidValue;
  
  if (val[7]=='w') h+=02;
  else if (val[7]!='-') return FSResult::InvalidValue;
  
  if (val[6]=='r') h+=04;
  else if (val[6]!='-') return FSResult::InvalidValue;
  
  if (val[5]=='x') h+=010;
  else if (val[5]=='s') h+=010+02000;
  else if (val[5]=='S') h+=02000;
  else if (val[5]!='-') return FSResult::InvalidValue;
  
  if (val[4]=='w') h+=020;
  else if (val[4]!='-') return FSResult::InvalidValue;
  
  if (val[3]=='r') h+=040;
  else if (val[3]!='-') return FSResult::InvalidValue;
  
  if (val[2]=='x') h+=0100;
  else if (val[2]=='s') h+=0100+04000;
  else if (val[2]=='S') h+=04000;
  else if (val[2]!='-') return FSResult::InvalidValue;
  
  if (val[1]=='w') h+=0200;
  else if (val[1]!='-') return FSResult::InvalidValue;
  
  if (val[0]=='r') h+=0400;
  else if (val[0]!='-') return FSResult::InvalidValue;
  perm=h;
  return FSResult::NoError;
}

/**
 * Converts a given permission into a string of the format rwxrwxrwx+sticky+setgid+setuid.
 * 
 * @param perm int value of given permissions
 * @param val buffer to store the string (will be modified by operation)
 * @param size size of the given buffer (will be modified by operation)
 * @return Returns the status of the operation as an FSResult instanze 
 *         (FSResult::NoError if no error occured or FSResult::BufferTooSmall 
 *         if buffer would exceed his max. size)
 */
FSResult perm2str(unsigned short perm,char *val, int &size) 
{
  if (size<10) return FSResult::BufferTooSmall;

  char wb[]="---------";
  if (perm&01) wb[8]='x';
  if (perm&02) wb[7]='w';
  if (perm&04) wb[6]='r';
  if (perm&010) wb[5]='x';
  if (perm&020) wb[4]='w';
  if (perm&040) wb[3]='r';
  if (perm&0100) wb[2]='x';
  if (perm&0200) wb[1]='w';
  if (perm&0400) wb[0]='r';
  // VTX 
  if (perm&01000)                                 // others==x
  { 
    if (perm&01) wb[8]='t'; else wb[8]='T';
  }
  
  // set gid
  if (perm&02000)                                 // group==x
  { 
    if (perm&010) wb[5]='s'; else wb[5]='S'; 
  }
  
  // set uid
  if (perm&04000)                                 // user==x
  { 
    if (perm&0100) wb[2]='s'; else wb[2]='S';
  }
  strcpy(val,wb);
  return FSResult::NoError;
}

/**
 * Isolates the type of the file from the mode and returns it as string
 * ('fifo', 'chardev', 'dir', 'blockdev', 'file', 'symlink', 'socket',
 * 'unknown).
 * 
 * @param mode mode of the file
 * @param val buffer to store the string (will be modified by operation)
 * @param size size of the given buffer (will be modified by operation)
 * @return Returns the status of the operation as an FSResult instanze 
 *         (FSResult::NoError if no error occured or FSResult::BufferTooSmall 
 *         if buffer would exceed his max. size)
 */
FSResult type2str(unsigned short mode, char *&val, int &size) 
{
  int i;
  char* wb[]={"fifo","chardev","dir","blockdev","file","symlink","socket","unknown"};
  switch (mode>>12)
  {
   case 01: i=0; break;
   case 02: i=1; break;
   case 04: i=2; break;
   case 06: i=3; break;
   case 010: i=4; break;
   case 012: i=5; break;
   case 014: i=6; break;
   default : i=7;
  }
  if ((size=snprintf(val,size,"%s",wb[i]))<0) return FSResult::BufferTooSmall;
  else return FSResult::NoError;
}
